# $Id: gfn_gen_table.gap,v 1.4 2006/02/04 13:15:56 maxhorn Exp $

# The parameters of our group
n := 3;
#q := 9;
F := GF(q^2);
z := PrimitiveRoot(F);
elem := Elements(F);;


#
# This file generates multiplication tables etc. for the
# arithmetic in the finite field of order q^2, to be used
# by our C(++) code.
#
file := Concatenation("gf", String(q^2), "_table.h");

PrintTo(file, "");

AppendTo(file, "#define Q ", q, "\n");
AppendTo(file, "#define N ", q^2, "\n\n");

AppendTo(file, "static const byte invTable[N] = {\n");
AppendTo(file, "0");
for j in [2..q^2] do
	AppendTo(file, ", ");
	AppendTo(file, Position(elem, 1/elem[j])-1);
od;
AppendTo(file, "\n");
AppendTo(file, "};\n\n");


AppendTo(file, "static const byte negTable[N] = {\n");
AppendTo(file, Position(elem, -elem[1])-1);
for j in [2..q^2] do
	AppendTo(file, ", ");
	AppendTo(file, Position(elem, -elem[j])-1);
od;
AppendTo(file, "\n");
AppendTo(file, "};\n\n");


AppendTo(file, "static const byte conjTable[N] = {\n");
AppendTo(file, Position(elem, elem[1]^q)-1);
for j in [2..q^2] do
	AppendTo(file, ", ");
	AppendTo(file, Position(elem, elem[j]^q)-1);
od;
AppendTo(file, "\n");
AppendTo(file, "};\n\n");


AppendTo(file, "static const byte mulTable[N][N] = {\n");
for i in [1..q^2] do
	AppendTo(file, "\t{ ");
	AppendTo(file, Position(elem, elem[i]*elem[1])-1);
	for j in [2..q^2] do
		AppendTo(file, ", ");
		AppendTo(file, Position(elem, elem[i]*elem[j])-1);
	od;
	AppendTo(file, " }");
	if i < q^2 then
		AppendTo(file, ",");
	fi;
	AppendTo(file, "\n");
od;
AppendTo(file, "};\n\n");


AppendTo(file, "static const byte addTable[N][N] = {\n");
for i in [1..q^2] do
	AppendTo(file, "\t{ ");
	AppendTo(file, Position(elem, elem[i]+elem[1])-1);
	for j in [2..q^2] do
		AppendTo(file, ", ");
		AppendTo(file, Position(elem, elem[i]+elem[j])-1);
	od;
	AppendTo(file, " }");
	if i < q^2 then
		AppendTo(file, ",");
	fi;
	AppendTo(file, "\n");
od;
AppendTo(file, "};\n\n");



##########################################
#
# SO(3,q^2)
#
##########################################

B := [
		[0, 1, 0],
		[1, 0, 0],
		[0, 0, 1]
	] * z^0;
G := SO(3,q^2);
A := InvariantBilinearForm(G).matrix;
AT := TransposedMat(Eigenvectors(F, A));
BT := TransposedMat(Eigenvectors(F, B));

y := (BT^-1 * B * TransposedMat(BT^-1));
x := (AT^-1 * A * TransposedMat(AT^-1));
DT := List(x/y, r -> List(r, c -> First(F, g -> g*g = c)));

T := AT * DT * BT^-1;

Assert(0, T * B * TransposedMat(T) = A);
SO3qq := G^T;
Assert(0, ForAll(GeneratorsOfGroup(SO3qq), g -> g * B * TransposedMat(g) = B));


##########################################
#
# SO(3,q)
#
##########################################

FrobeniusToMatrix := function ( M, q )
	Assert(0, IsMatrix(M));
	return List(M, row -> List(row,x -> x^q));
end;;


SO3q := Group(IdentityMat(3, F));
while Size(SO3q) < Size(SO(3,q)) do
	repeat
		g := PseudoRandom(SO3qq);
	until TransposedMat(g)^-1 = FrobeniusToMatrix(g, q);
	SO3q := ClosureGroup(SO3q, g);
	Size(SO3q);
od;


##########################################
orbs := OrbitsDomain(SO3q, F^3);;
orbReps := List(orbs, x -> x[1]);;



AppendTo(file, "static const byte orbReps[Q*(Q*(Q+1)+1)][3] = {\n");
for i in [1..Length(orbReps)] do
	x := orbReps[i];
	AppendTo(file, "\t{ ");
	AppendTo(file, Position(elem, x[1])-1);
	AppendTo(file, ", ");
	AppendTo(file, Position(elem, x[2])-1);
	AppendTo(file, ", ");
	AppendTo(file, Position(elem, x[3])-1);
	AppendTo(file, " }");
	if i < Length(orbReps) then
		AppendTo(file, ",");
	fi;
	AppendTo(file, "\n");
od;
AppendTo(file, "};\n\n");

